@@ -14,7 +14,8 @@ const Tweener = imports.ui.tweener;
 
 const POPUP_APPICON_SIZE = 96;
 const POPUP_SCROLL_TIME = 0.10; // seconds
-const POPUP_FADE_TIME = 0.1; // seconds
+const POPUP_FADE_IN_TIME = 0.4; // seconds
+const POPUP_FADE_OUT_TIME = 0.1; // seconds
 
 const APP_ICON_HOVER_TIMEOUT = 200; // milliseconds
 
@@ -74,7 +75,7 @@ AltTabPopup.prototype = {
 
     _allocate: function (actor, box, flags) {
         let childBox = new Clutter.ActorBox();
-        let primary = global.get_primary_monitor();
+        let primary = Main.layoutManager.primaryMonitor;
 
         let leftPadding = this.actor.get_theme_node().get_padding(St.Side.LEFT);
         let rightPadding = this.actor.get_theme_node().get_padding(St.Side.RIGHT);
@@ -119,7 +120,7 @@ AltTabPopup.prototype = {
         }
     },
 
-    show : function(backward, switch_group) {
+    show : function(backward, binding) {
         let tracker = Shell.WindowTracker.get_default();
         let apps = tracker.get_running_apps ('');
 
@@ -150,7 +151,7 @@ AltTabPopup.prototype = {
         this.actor.get_allocation_box();
 
         // Make the initial selection
-        if (switch_group) {
+        if (binding == 'switch_group') {
             if (backward) {
                 this._select(0, this._appIcons[0].cachedWindows.length - 1);
             } else {
@@ -159,6 +160,10 @@ AltTabPopup.prototype = {
                 else
                     this._select(0, 0);
             }
+        } else if (binding == 'switch_group_backward') {
+            this._select(0, this._appIcons[0].cachedWindows.length - 1);
+        } else if (binding == 'switch_windows_backward') {
+            this._select(this._appIcons.length - 1);
         } else if (this._appIcons.length == 1) {
             this._select(0);
         } else if (backward) {
@@ -178,10 +183,15 @@ AltTabPopup.prototype = {
             return false;
         }
 
+        // Using easeInOutExpo over 400ms gives us 150ms of "nearly
+        // invisible" (less than 10% opacity), followed by a 100ms
+        // tween in (to 90% opacity, with the last 10% coming over the
+        // next 150ms). So if the user releases Alt quickly after we
+        // start tweening, they'll never see the switcher.
         Tweener.addTween(this.actor,
                          { opacity: 255,
-                           time: POPUP_FADE_TIME,
-                           transition: 'easeOutQuad'
+                           time: POPUP_FADE_IN_TIME,
+                           transition: 'easeInOutExpo'
                          });
 
         return true;
@@ -221,8 +231,12 @@ AltTabPopup.prototype = {
             this.destroy();
         } else if (action == Meta.KeyBindingAction.SWITCH_GROUP) {
             this._select(this._currentApp, backwards ? this._previousWindow() : this._nextWindow());
+        } else if (action == Meta.KeyBindingAction.SWITCH_GROUP_BACKWARD) {
+            this._select(this._currentApp, this._previousWindow());
         } else if (action == Meta.KeyBindingAction.SWITCH_WINDOWS) {
             this._select(backwards ? this._previousApp() : this._nextApp());
+        } else if (action == Meta.KeyBindingAction.SWITCH_WINDOWS_BACKWARD) {
+            this._select(this._previousApp());
         } else if (this._thumbnailsFocused) {
             if (keysym == Clutter.Left)
                 this._select(this._currentApp, this._previousWindow());
@@ -360,7 +374,7 @@ AltTabPopup.prototype = {
         if (this.actor.visible) {
             Tweener.addTween(this.actor,
                              { opacity: 0,
-                               time: POPUP_FADE_TIME,
+                               time: POPUP_FADE_OUT_TIME,
                                transition: 'easeOutQuad',
                                onComplete: Lang.bind(this,
                                    function() {
@@ -618,7 +632,7 @@ SwitcherList.prototype = {
                 this._items[this._highlighted].add_style_pseudo_class('selected');
         }
 
-        let monitor = global.get_primary_monitor();
+        let monitor = Main.layoutManager.primaryMonitor;
         let itemSize = this._items[index].allocation.x2 - this._items[index].allocation.x1;
         let [posX, posY] = this._items[index].get_transformed_position();
         posX += this.actor.x;
@@ -646,7 +660,7 @@ SwitcherList.prototype = {
 
     _scrollToRight : function() {
         this._scrollableLeft = true;
-        let monitor = global.get_primary_monitor();
+        let monitor = Main.layoutManager.primaryMonitor;
         let padding = this.actor.get_theme_node().get_horizontal_padding();
         let parentPadding = this.actor.get_parent().get_theme_node().get_horizontal_padding();
         let x = this._items[this._highlighted].allocation.x2 - monitor.width + padding + parentPadding;
@@ -743,7 +757,7 @@ SwitcherList.prototype = {
         let children = this._list.get_children();
         let childBox = new Clutter.ActorBox();
 
-        let primary = global.get_primary_monitor();
+        let primary = Main.layoutManager.primaryMonitor;
         let parentRightPadding = this.actor.get_parent().get_theme_node().get_padding(St.Side.RIGHT);
         if (this.actor.allocation.x2 == primary.x + primary.width - parentRightPadding) {
             if (this._squareItems)
@@ -873,7 +887,7 @@ AppSwitcher.prototype = {
            totalSpacing += this._separator.width + this._list.spacing;
 
         // We just assume the whole screen here due to weirdness happing with the passed width
-        let primary = global.get_primary_monitor();
+        let primary = Main.layoutManager.primaryMonitor;
         let parentPadding = this.actor.get_parent().get_theme_node().get_horizontal_padding();
         let availWidth = primary.width - parentPadding - this.actor.get_theme_node().get_horizontal_padding();
         let height = 0;
