@@ -2,9 +2,11 @@
 
 const Clutter = imports.gi.Clutter;
 const Lang = imports.lang;
+const Meta = imports.gi.Meta;
 const St = imports.gi.St;
 const Shell = imports.gi.Shell;
 
+const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 
 const POPUP_ANIMATION_TIME = 0.15;
@@ -27,7 +29,6 @@ BoxPointer.prototype = {
     _init: function(arrowSide, binProperties) {
         this._arrowSide = arrowSide;
         this._arrowOrigin = 0;
-        this._arrowCorner = null;
         this.actor = new St.Bin({ x_fill: true,
                                   y_fill: true });
         this._container = new Shell.GenericContainer();
@@ -41,80 +42,80 @@ BoxPointer.prototype = {
         this._border.connect('repaint', Lang.bind(this, this._drawBorder));
         this._container.add_actor(this._border);
         this.bin.raise(this._border);
+        this._xOffset = 0;
+        this._yOffset = 0;
+        this._xPosition = 0;
+        this._yPosition = 0;
     },
 
     show: function(animate, onComplete) {
-        let x = this.actor.x;
-        let y = this.actor.y;
         let themeNode = this.actor.get_theme_node();
         let rise = themeNode.get_length('-arrow-rise');
 
-        this.actor.opacity = 0;
+        this.opacity = 0;
         this.actor.show();
 
         if (animate) {
             switch (this._arrowSide) {
                 case St.Side.TOP:
-                    this.actor.y -= rise;
+                    this.yOffset = -rise;
                     break;
                 case St.Side.BOTTOM:
-                    this.actor.y += rise;
+                    this.yOffset = rise;
                     break;
                 case St.Side.LEFT:
-                    this.actor.x -= rise;
+                    this.xOffset = -rise;
                     break;
                 case St.Side.RIGHT:
-                    this.actor.x += rise;
+                    this.xOffset = rise;
                     break;
             }
         }
 
-        Tweener.addTween(this.actor, { opacity: 255,
-                                       x: x,
-                                       y: y,
-                                       transition: "linear",
-                                       onComplete: onComplete,
-                                       time: POPUP_ANIMATION_TIME });
+        Tweener.addTween(this, { opacity: 255,
+                                 xOffset: 0,
+                                 yOffset: 0,
+                                 transition: "linear",
+                                 onComplete: onComplete,
+                                 time: POPUP_ANIMATION_TIME });
     },
 
     hide: function(animate, onComplete) {
-        let x = this.actor.x;
-        let y = this.actor.y;
-        let originalX = this.actor.x;
-        let originalY = this.actor.y;
+        let xOffset = 0;
+        let yOffset = 0;
         let themeNode = this.actor.get_theme_node();
         let rise = themeNode.get_length('-arrow-rise');
 
         if (animate) {
             switch (this._arrowSide) {
                 case St.Side.TOP:
-                    y += rise;
+                    yOffset = rise;
                     break;
                 case St.Side.BOTTOM:
-                    y -= rise;
+                    yOffset = -rise;
                     break;
                 case St.Side.LEFT:
-                    x += rise;
+                    xOffset = rise;
                     break;
                 case St.Side.RIGHT:
-                    x -= rise;
+                    xOffset = -rise;
                     break;
             }
         }
 
-        Tweener.addTween(this.actor, { opacity: 0,
-                                       x: x,
-                                       y: y,
-                                       transition: "linear",
-                                       time: POPUP_ANIMATION_TIME,
-                                       onComplete: Lang.bind(this, function () {
-                                           this.actor.hide();
-                                           this.actor.x = originalX;
-                                           this.actor.y = originalY;
-                                           if (onComplete)
-                                               onComplete();
-                                       })
-                                     });
+        Tweener.addTween(this, { opacity: 0,
+                                 xOffset: xOffset,
+                                 yOffset: yOffset,
+                                 transition: "linear",
+                                 time: POPUP_ANIMATION_TIME,
+                                 onComplete: Lang.bind(this, function () {
+                                     this.actor.hide();
+                                     this.xOffset = 0;
+                                     this.yOffset = 0;
+                                     if (onComplete)
+                                         onComplete();
+                                 })
+                               });
     },
 
     _adjustAllocationForArrow: function(isWidth, alloc) {
@@ -177,6 +178,9 @@ BoxPointer.prototype = {
                 break;
         }
         this.bin.allocate(childBox, flags);
+
+        if (this._sourceActor && this._sourceActor.mapped)
+            this._reposition(this._sourceActor, this._gap, this._alignment);
     },
 
     _drawBorder: function(area) {
@@ -190,10 +194,8 @@ BoxPointer.prototype = {
         let halfBorder = borderWidth / 2;
         let halfBase = Math.floor(base/2);
 
-        let borderColor = new Clutter.Color();
-        themeNode.get_color('-arrow-border-color', borderColor);
-        let backgroundColor = new Clutter.Color();
-        themeNode.get_color('-arrow-background-color', backgroundColor);
+        let borderColor = themeNode.get_color('-arrow-border-color');
+        let backgroundColor = themeNode.get_color('-arrow-background-color');
 
         let [width, height] = area.get_surface_size();
         let [boxWidth, boxHeight] = [width, height];
@@ -218,135 +220,84 @@ BoxPointer.prototype = {
 
         cr.moveTo(x1 + borderRadius, y1);
         if (this._arrowSide == St.Side.TOP) {
-            if (this._arrowCorner == St.Corner.TOPLEFT) {
-                cr.moveTo(x1, y1);
-                cr.lineTo(x1, y1 - rise);
-                cr.lineTo(x1 + halfBase, y1);
-                cr.lineTo(x2 - borderRadius, y1);
-            } else if (this._arrowCorner == St.Corner.TOPRIGHT) {
-                cr.lineTo(x2 - halfBase, y1);
-                cr.lineTo(x2, y1 - rise);
-            } else if (this._arrowOrigin < (x1 + (borderRadius + halfBase))) {
-                cr.lineTo(this._arrowOrigin, y1);
+            if (this._arrowOrigin < (x1 + (borderRadius + halfBase))) {
                 cr.lineTo(this._arrowOrigin, y1 - rise);
-                cr.lineTo(this._arrowOrigin + halfBase, y1);
-                cr.lineTo(x2 - borderRadius, y1);
+                cr.lineTo(Math.max(x1 + borderRadius, this._arrowOrigin) + halfBase, y1);
             } else if (this._arrowOrigin > (x2 - (borderRadius + halfBase))) {
-                cr.lineTo(this._arrowOrigin - halfBase, y1);
+                cr.lineTo(Math.min(x2 - borderRadius, this._arrowOrigin) - halfBase, y1);
                 cr.lineTo(this._arrowOrigin, y1 - rise);
-                cr.lineTo(this._arrowOrigin, y1);
-                cr.lineTo(x2 - borderRadius, y1);
             } else {
                 cr.lineTo(this._arrowOrigin - halfBase, y1);
                 cr.lineTo(this._arrowOrigin, y1 - rise);
                 cr.lineTo(this._arrowOrigin + halfBase, y1);
-                cr.lineTo(x2 - borderRadius, y1);
             }
-        } else
-            cr.lineTo(x2 - borderRadius, y1);
+        }
+
+        cr.lineTo(x2 - borderRadius, y1);
 
         // top-right corner
-        if (this._arrowCorner != St.Corner.TOPRIGHT)
-            cr.arc(x2 - borderRadius, y1 + borderRadius, borderRadius,
-                   3*Math.PI/2, Math.PI*2);
+        cr.arc(x2 - borderRadius, y1 + borderRadius, borderRadius,
+               3*Math.PI/2, Math.PI*2);
 
         if (this._arrowSide == St.Side.RIGHT) {
-            if (this._arrowCorner == St.Corner.TOPRIGHT) {
-                cr.lineTo(x2, y1);
-                cr.lineTo(x2 + rise, y1);
-                cr.lineTo(x2, y1 + halfBase);
-                cr.lineTo(x2, y2 - borderRadius);
-            } else if (this._arrowCorner == St.Corner.BOTTOMRIGHT) {
-                cr.moveTo(x2, y2 - halfBase);
-                cr.lineTo(x2 + rise, y2);
-            } else if (this._arrowOrigin < (y1 + (borderRadius + halfBase))) {
-                cr.lineTo(x2, this._arrowOrigin);
+            if (this._arrowOrigin < (y1 + (borderRadius + halfBase))) {
                 cr.lineTo(x2 + rise, this._arrowOrigin);
-                cr.lineTo(x2, this._arrowOrigin + halfBase);
-                cr.lineTo(x2, y2 - borderRadius);
+                cr.lineTo(x2, Math.max(y1 + borderRadius, this._arrowOrigin) + halfBase);
             } else if (this._arrowOrigin > (y2 - (borderRadius + halfBase))) {
-                cr.lineTo(x2, this._arrowOrigin - halfBase);
+                cr.lineTo(x2, Math.min(y2 - borderRadius, this._arrowOrigin) - halfBase);
                 cr.lineTo(x2 + rise, this._arrowOrigin);
-                cr.lineTo(x2, this._arrowOrigin);
-                cr.lineTo(x2, y2 - borderRadius);
             } else {
                 cr.lineTo(x2, this._arrowOrigin - halfBase);
                 cr.lineTo(x2 + rise, this._arrowOrigin);
                 cr.lineTo(x2, this._arrowOrigin + halfBase);
-                cr.lineTo(x2, y2 - borderRadius);
             }
-        } else
-            cr.lineTo(x2, y2 - borderRadius);
+        }
+
+        cr.lineTo(x2, y2 - borderRadius);
 
         // bottom-right corner
-        if (this._arrowCorner != St.Corner.BOTTOMRIGHT)
-            cr.arc(x2 - borderRadius, y2 - borderRadius, borderRadius,
-                   0, Math.PI/2);
+        cr.arc(x2 - borderRadius, y2 - borderRadius, borderRadius,
+               0, Math.PI/2);
 
         if (this._arrowSide == St.Side.BOTTOM) {
-            if (this._arrowCorner == St.Corner.BOTTOMLEFT) {
-                cr.lineTo(x1 + halfBase, y2);
-                cr.lineTo(x1, y2 + rise);
-            } else if (this._arrowCorner == St.Corner.BOTTOMRIGHT) {
-                cr.lineTo(x2, y2 + rise);
-                cr.lineTo(x2 - halfBase, y2);
-                cr.lineTo(x1 + borderRadius, y2);
-            } else if (this._arrowOrigin < (x1 + (borderRadius + halfBase))) {
-                cr.lineTo(this._arrowOrigin + halfBase, y2);
+            if (this._arrowOrigin < (x1 + (borderRadius + halfBase))) {
+                cr.lineTo(Math.max(x1 + borderRadius, this._arrowOrigin) + halfBase, y2);
                 cr.lineTo(this._arrowOrigin, y2 + rise);
-                cr.lineTo(this._arrowOrigin, y2);
-                cr.lineTo(x1 + borderRadius, y2);
             } else if (this._arrowOrigin > (x2 - (borderRadius + halfBase))) {
-                cr.lineTo(this._arrowOrigin, y2);
                 cr.lineTo(this._arrowOrigin, y2 + rise);
-                cr.lineTo(this._arrowOrigin - halfBase, y2);
-                cr.lineTo(x1 + borderRadius, y2);
+                cr.lineTo(Math.min(x2 - borderRadius, this._arrowOrigin) - halfBase, y2);
             } else {
                 cr.lineTo(this._arrowOrigin + halfBase, y2);
                 cr.lineTo(this._arrowOrigin, y2 + rise);
                 cr.lineTo(this._arrowOrigin - halfBase, y2);
-                cr.lineTo(x1 + borderRadius, y2);
             }
-        } else
-            cr.lineTo(x1 + borderRadius, y2);
+        }
+
+        cr.lineTo(x1 + borderRadius, y2);
 
         // bottom-left corner
-        if (this._arrowCorner != St.Corner.BOTTOMLEFT)
-            cr.arc(x1 + borderRadius, y2 - borderRadius, borderRadius,
-                   Math.PI/2, Math.PI);
+        cr.arc(x1 + borderRadius, y2 - borderRadius, borderRadius,
+               Math.PI/2, Math.PI);
 
         if (this._arrowSide == St.Side.LEFT) {
-            if (this._arrowCorner == St.Corner.TOPLEFT) {
-                cr.lineTo(x2, y1 + halfBase);
-                cr.lineTo(x1 - rise, y1);
-            } else if (this._arrowCorner == St.Corner.BOTTOMLEFT) {
-                cr.lineTo(x1 + rise, y2);
-                cr.moveTo(x1, y2 - halfBase);
-            } else if (this._arrowOrigin < (y1 + (borderRadius + halfBase))) {
-                cr.lineTo(x1, this._arrowOrigin + halfBase);
+            if (this._arrowOrigin < (y1 + (borderRadius + halfBase))) {
+                cr.lineTo(x1, Math.max(y1 + borderRadius, this._arrowOrigin) + halfBase);
                 cr.lineTo(x1 - rise, this._arrowOrigin);
-                cr.lineTo(x1, this._arrowOrigin);
-                cr.lineTo(x1, y1 + borderRadius);
             } else if (this._arrowOrigin > (y2 - (borderRadius + halfBase))) {
-                cr.lineTo(x1, this._arrowOrigin);
                 cr.lineTo(x1 - rise, this._arrowOrigin);
-                cr.lineTo(x1, this._arrowOrigin - halfBase);
-                cr.lineTo(x1, y1 + borderRadius);
+                cr.lineTo(x1, Math.min(y2 - borderRadius, this._arrowOrigin) - halfBase);
             } else {
                 cr.lineTo(x1, this._arrowOrigin + halfBase);
                 cr.lineTo(x1 - rise, this._arrowOrigin);
                 cr.lineTo(x1, this._arrowOrigin - halfBase);
-                cr.lineTo(x1, y1 + borderRadius);
             }
-        } else
-            cr.lineTo(x1, y1 + borderRadius);
+        }
+
+        cr.lineTo(x1, y1 + borderRadius);
 
         // top-left corner
-        if (this._arrowCorner != St.Corner.TOPLEFT)
-            cr.arc(x1 + borderRadius, y1 + borderRadius, borderRadius,
-                   Math.PI, 3*Math.PI/2);
-        else
-            cr.lineTo(x1, y1);
+        cr.arc(x1 + borderRadius, y1 + borderRadius, borderRadius,
+               Math.PI, 3*Math.PI/2);
 
         Clutter.cairo_set_source_color(cr, backgroundColor);
         cr.fillPreserve();
@@ -360,38 +311,47 @@ BoxPointer.prototype = {
         // so that we can query the correct size.
         this.actor.show();
 
+        this._sourceActor = sourceActor;
+        this._gap = gap;
+        this._alignment = alignment;
+
+        this._reposition(sourceActor, gap, alignment);
+    },
+
+    _reposition: function(sourceActor, gap, alignment) {
         // Position correctly relative to the sourceActor
-        let [sourceX, sourceY] = sourceActor.get_transformed_position();
-        let [sourceWidth, sourceHeight] = sourceActor.get_transformed_size();
-        let [sourceCenterX, sourceCenterY] = [sourceX + (sourceWidth / 2), sourceY + (sourceHeight / 2)];
+        let sourceNode = sourceActor.get_theme_node();
+        let sourceContentBox = sourceNode.get_content_box(sourceActor.get_allocation_box());
+        let sourceAllocation = Shell.util_get_transformed_allocation(sourceActor);
+        let sourceCenterX = sourceAllocation.x1 + sourceContentBox.x1 + (sourceContentBox.x2 - sourceContentBox.x1) / 2;
+        let sourceCenterY = sourceAllocation.y1 + sourceContentBox.y1 + (sourceContentBox.y2 - sourceContentBox.y1) / 2;
         let [minWidth, minHeight, natWidth, natHeight] = this.actor.get_preferred_size();
 
         // We also want to keep it onscreen, and separated from the
         // edge by the same distance as the main part of the box is
         // separated from its sourceActor
-        let primary = global.get_primary_monitor();
+        let primary = Main.layoutManager.primaryMonitor;
         let themeNode = this.actor.get_theme_node();
-        let halfBorder = themeNode.get_length('-arrow-border-width') / 2;
-        let halfBase = themeNode.get_length('-arrow-base') / 2;
+        let borderWidth = themeNode.get_length('-arrow-border-width');
+        let arrowBase = themeNode.get_length('-arrow-base');
         let borderRadius = themeNode.get_length('-arrow-border-radius');
-
-        let margin = 2 * borderRadius + halfBorder;
+        let margin = (4 * borderRadius + borderWidth + arrowBase);
+        let halfMargin = margin / 2;
 
         let resX, resY;
-        this._arrowCorner = null;
 
         switch (this._arrowSide) {
         case St.Side.TOP:
-            resY = sourceY + sourceHeight + gap;
+            resY = sourceAllocation.y2 + gap;
             break;
         case St.Side.BOTTOM:
-            resY = sourceY - natHeight - gap;
+            resY = sourceAllocation.y1 - natHeight - gap;
             break;
         case St.Side.LEFT:
-            resX = sourceX + sourceWidth + gap;
+            resX = sourceAllocation.x2 + gap;
             break;
         case St.Side.RIGHT:
-            resX = sourceX - natWidth - gap;
+            resX = sourceAllocation.x1 - natWidth - gap;
             break;
         }
 
@@ -400,59 +360,19 @@ BoxPointer.prototype = {
         switch (this._arrowSide) {
         case St.Side.TOP:
         case St.Side.BOTTOM:
-            switch (alignment) {
-            case St.Align.START:
-                resX = sourceCenterX - (halfBase + borderRadius + halfBorder);
-                break;
-            case St.Align.MIDDLE:
-                resX = sourceCenterX - (natWidth / 2);
-                break;
-            case St.Align.END:
-                resX = sourceCenterX - natWidth + (halfBase + borderRadius + halfBorder);
-                break;
-            }
-            if (sourceCenterX < margin) {
-                // Not enough space to the top
-                this._arrowCorner = (this._arrowSide == St.Side.TOP) ? St.Corner.TOPLEFT : St.Corner.BOTTOMLEFT;
-                resX = 10;
-            } else if (sourceCenterX > (primary.width - margin)) {
-                // Not enough space to the botom
-                this._arrowCorner = (this._arrowSide == St.Side.TOP) ? St.Corner.TOPRIGHT : St.Corner.BOTTOMRIGHT;
-                resX = primary.width - (10 + natWidth);
-            }
-
-            resX = Math.max(resX, 10);
-            resX = Math.min(resX, primary.width - (10 + natWidth));
+            resX = sourceCenterX - (halfMargin + (natWidth - margin) * alignment);
 
+            resX = Math.max(resX, primary.x + 10);
+            resX = Math.min(resX, primary.x + primary.width - (10 + natWidth));
             this.setArrowOrigin(sourceCenterX - resX);
             break;
 
         case St.Side.LEFT:
         case St.Side.RIGHT:
-            switch (alignment) {
-            case St.Align.START:
-                resY = sourceCenterY - (halfBase + borderRadius + halfBorder);
-                break;
-            case St.Align.MIDDLE:
-                resY = sourceCenterY - (natHeight / 2);
-                break;
-            case St.Align.END:
-                resY = sourceCenterY - natHeight + (halfBase + borderRadius + halfBorder);
-                break;
-            }
-            if (sourceCenterY < margin) {
-                // Not enough space to the left
-                this._arrowCorner = (this._arrowSide == St.Side.LEFT) ? St.Corner.TOPLEFT : St.Corner.TORIGHT;
-                resY = 10;
-            }
-            else if (sourceCenterY > (primary.height - margin)) {
-                // Not enough space to the right
-                this._arrowCorner = (this._arrowSide == St.Side.LEFT) ? St.Corner.BOTTOMLEFT : St.Corner.BOTTOMRIGHT;
-                resY = primary.height - (10 + natHeight);
-            }
+            resY = sourceCenterY - (halfMargin + (natHeight - margin) * alignment);
 
-            resY = Math.max(resY, 10);
-            resY = Math.min(resY, primary.height - (10 + natHeight));
+            resY = Math.max(resY, primary.y + 10);
+            resY = Math.min(resY, primary.y + primary.height - (10 + natHeight));
 
             this.setArrowOrigin(sourceCenterY - resY);
             break;
@@ -465,9 +385,9 @@ BoxPointer.prototype = {
             parent = parent.get_parent();
         }
 
-        // Actually set the position
-        this.actor.x = Math.floor(x);
-        this.actor.y = Math.floor(y);
+        this._xPosition = Math.floor(x);
+        this._yPosition = Math.floor(y);
+        this._shiftActor();
     },
 
     // @origin: Coordinate specifying middle of the arrow, along
@@ -478,5 +398,42 @@ BoxPointer.prototype = {
             this._arrowOrigin = origin;
             this._border.queue_repaint();
         }
+    },
+
+    _shiftActor : function() {
+        // Since the position of the BoxPointer depends on the allocated size
+        // of the BoxPointer and the position of the source actor, trying
+        // to position the BoxPoiner via the x/y properties will result in
+        // allocation loops and warnings. Instead we do the positioning via
+        // the anchor point, which is independent of allocation, and leave
+        // x == y == 0.
+        this.actor.set_anchor_point(-(this._xPosition + this._xOffset),
+                                    -(this._yPosition + this._yOffset));
+    },
+
+    set xOffset(offset) {
+        this._xOffset = offset;
+        this._shiftActor();
+    },
+
+    get xOffset() {
+        return this._xOffset;
+    },
+
+    set yOffset(offset) {
+        this._yOffset = offset;
+        this._shiftActor();
+    },
+
+    get yOffset() {
+        return this._yOffset;
+    },
+
+    set opacity(opacity) {
+        this.actor.opacity = opacity;
+    },
+
+    get opacity() {
+        return this.actor.opacity;
     }
 };
