@@ -6,8 +6,6 @@ const Lang = imports.lang;
 const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
 const St = imports.gi.St;
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
 
 const AppDisplay = imports.ui.appDisplay;
 const AppFavorites = imports.ui.appFavorites;
@@ -262,11 +260,6 @@ Dash.prototype = {
                                        clip_to_allocation: true });
         this._box._delegate = this;
 
-        // This will eventually be automatic, see
-        // https://bugzilla.gnome.org/show_bug.cgi?id=584662
-        if (St.Widget.get_default_direction () == St.TextDirection.RTL)
-            this._box.add_style_pseudo_class('rtl');
-
         this.actor = new St.Bin({ y_align: St.Align.START, child: this._box });
         this.actor.connect('notify::height', Lang.bind(this,
             function() {
@@ -288,20 +281,37 @@ Dash.prototype = {
                               Lang.bind(this, this._onDragBegin));
         Main.overview.connect('item-drag-end',
                               Lang.bind(this, this._onDragEnd));
+        Main.overview.connect('item-drag-cancelled',
+                              Lang.bind(this, this._onDragCancelled));
         Main.overview.connect('window-drag-begin',
                               Lang.bind(this, this._onDragBegin));
+        Main.overview.connect('window-drag-cancelled',
+                              Lang.bind(this, this._onDragCancelled));
         Main.overview.connect('window-drag-end',
                               Lang.bind(this, this._onDragEnd));
     },
 
     _onDragBegin: function() {
+        this._dragCancelled = false;
         this._dragMonitor = {
             dragMotion: Lang.bind(this, this._onDragMotion)
         };
         DND.addDragMonitor(this._dragMonitor);
     },
 
+    _onDragCancelled: function() {
+        this._dragCancelled = true;
+        this._endDrag();
+    },
+
     _onDragEnd: function() {
+        if (this._dragCancelled)
+            return;
+
+        this._endDrag();
+    },
+
+    _endDrag: function() {
         this._clearDragPlaceholder();
         if (this._favRemoveTarget) {
             this._favRemoveTarget.actor.hide();
@@ -314,7 +324,7 @@ Dash.prototype = {
                     this._favRemoveTarget = null;
                 }));
         }
-        DND.removeMonitor(this._dragMonitor);
+        DND.removeDragMonitor(this._dragMonitor);
     },
 
     _onDragMotion: function(dragEvent) {
@@ -332,7 +342,10 @@ Dash.prototype = {
 
         let srcIsFavorite = (id in favorites);
 
-        if (srcIsFavorite && this._favRemoveTarget == null) {
+        if (srcIsFavorite &&
+            dragEvent.source.actor &&
+            this.actor.contains (dragEvent.source.actor) &&
+            this._favRemoveTarget == null) {
                 this._favRemoveTarget = new RemoveFavoriteIcon();
                 this._favRemoveTarget.icon.setIconSize(this.iconSize);
                 this._box.add(this._favRemoveTarget.actor);
@@ -377,6 +390,8 @@ Dash.prototype = {
                                    Lang.bind(this, function() {
                                        display.actor.opacity = 255;
                                    }));
+        display.actor.set_tooltip_text(app.get_name());
+
         let item = new DashItemContainer();
         item.setChild(display.actor);
 
