@@ -22,9 +22,6 @@ const DBus = imports.dbus;
 const Lang = imports.lang;
 const Signals = imports.signals;
 
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
-
 const Clutter = imports.gi.Clutter;
 const Gdm = imports.gi.Gdm;
 const GLib = imports.gi.GLib;
@@ -366,8 +363,10 @@ EndSessionDialog.prototype = {
 
         if (this._user.is_loaded && !dialogContent.iconName) {
             let iconFile = this._user.get_icon_file();
-
-            this._setIconFromFile(iconFile, dialogContent.iconStyleClass);
+            if (GLib.file_test(iconFile, GLib.FileTest.EXISTS))
+                this._setIconFromFile(iconFile, dialogContent.iconStyleClass);
+            else
+                this._setIconFromName('avatar-default', dialogContent.iconStyleClass);
         } else if (dialogContent.iconName) {
             this._setIconFromName(dialogContent.iconName,
                                   dialogContent.iconStyleClass);
@@ -406,10 +405,6 @@ EndSessionDialog.prototype = {
     },
 
     _updateButtons: function() {
-        if (this.state != ModalDialog.State.OPENING &&
-            this.state != ModalDialog.State.OPENED)
-            return;
-
         let dialogContent = DialogContent[this._type];
         let buttons = [{ action: Lang.bind(this, this.cancel),
                          label:  _("Cancel"),
@@ -519,11 +514,12 @@ EndSessionDialog.prototype = {
             this._inhibitors.push(inhibitor);
         }
 
+        this._updateButtons();
+
         if (!this.open(timestamp))
             throw new DBus.DBusError('org.gnome.Shell.ModalDialog.GrabError',
                                      "Cannot grab pointer and keyboard");
 
-        this._updateButtons();
         this._updateContent();
 
         let signalId = this.connect('opened',
