@@ -10,8 +10,6 @@ const Pango = imports.gi.Pango;
 const St = imports.gi.St;
 const Shell = imports.gi.Shell;
 const Signals = imports.signals;
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
 
 const Params = imports.misc.params;
 
@@ -39,13 +37,16 @@ ModalDialog.prototype = {
         params = Params.parse(params, { styleClass: null });
 
         this.state = State.CLOSED;
+        this._hasModal = false;
 
         this._group = new St.Group({ visible: false,
                                      x: 0,
                                      y: 0 });
         Main.uiGroup.add_actor(this._group);
-        global.focus_manager.add_group(this._group);
-        this._initialKeyFocus = this._group;
+
+        let constraint = new Clutter.BindConstraint({ source: global.stage,
+                                                      coordinate: Clutter.BindCoordinate.POSITION | Clutter.BindCoordinate.SIZE });
+        this._group.add_constraint(constraint);
 
         this._group.connect('destroy', Lang.bind(this, this._onGroupDestroy));
 
@@ -60,12 +61,18 @@ ModalDialog.prototype = {
         this._group.add_actor(this._backgroundBin);
         this._lightbox.highlight(this._backgroundBin);
 
+        this._backgroundStack = new Shell.Stack();
+        this._backgroundBin.child = this._backgroundStack;
+
+        this._eventBlocker = new Clutter.Group({ reactive: true });
+        this._backgroundStack.add_actor(this._eventBlocker);
+
         this._dialogLayout = new St.BoxLayout({ style_class: 'modal-dialog',
                                                 vertical:    true });
         if (params.styleClass != null) {
             this._dialogLayout.add_style_class_name(params.styleClass);
         }
-        this._backgroundBin.child = this._dialogLayout;
+        this._backgroundStack.add_actor(this._dialogLayout);
 
         this.contentLayout = new St.BoxLayout({ vertical: true });
         this._dialogLayout.add(this.contentLayout,
@@ -74,12 +81,17 @@ ModalDialog.prototype = {
                                  x_align: St.Align.MIDDLE,
                                  y_align: St.Align.START });
 
-        this._buttonLayout = new St.BoxLayout({ opacity:  220,
-                                                vertical: false });
+        this._buttonLayout = new St.BoxLayout({ style_class: 'modal-dialog-button-box',
+                                                opacity:     220,
+                                                vertical:    false });
         this._dialogLayout.add(this._buttonLayout,
                                { expand:  true,
                                  x_align: St.Align.MIDDLE,
                                  y_align: St.Align.END });
+
+        global.focus_manager.add_group(this._dialogLayout);
+        this._initialKeyFocus = this._dialogLayout;
+        this._savedKeyFocus = null;
     },
 
     setButtons: function(buttons) {
@@ -137,7 +149,7 @@ ModalDialog.prototype = {
     },
 
     _fadeOpen: function() {
-        let monitor = global.get_focus_monitor();
+        let monitor = Main.layoutManager.focusMonitor;
 
         this._backgroundBin.set_position(monitor.x, monitor.y);
         this._backgroundBin.set_size(monitor.width, monitor.height);
@@ -154,22 +166,23 @@ ModalDialog.prototype = {
                            transition: 'easeOutQuad',
                            onComplete: Lang.bind(this,
                                function() {
-                                   this._initialKeyFocus.grab_key_focus();
                                    this.state = State.OPENED;
                                    this.emit('opened');
-                               }),
+                               })
                          });
     },
 
+    setInitialKeyFocus: function(actor) {
+        this._initialKeyFocus = actor;
+    },
+
     open: function(timestamp) {
         if (this.state == State.OPENED || this.state == State.OPENING)
             return true;
 
-        if (!Main.pushModal(this._group, timestamp))
+        if (!this.pushModal(timestamp))
             return false;
 
-        global.stage.set_key_focus(this._group);
-
         this._fadeOpen();
         return true;
     },
@@ -178,14 +191,9 @@ ModalDialog.prototype = {
         if (this.state == State.CLOSED || this.state == State.CLOSING)
             return;
 
-        let needsPopModal;
-
-        if (this.state == State.OPENED || this.state == State.OPENING)
-            needsPopModal = true;
-        else
-            needsPopModal = false;
-
         this.state = State.CLOSING;
+        this.popModal(timestamp);
+        this._savedKeyFocus = null;
 
         Tweener.addTween(this._group,
                          { opacity: 0,
@@ -195,13 +203,46 @@ ModalDialog.prototype = {
                                function() {
                                    this.state = State.CLOSED;
                                    this._group.hide();
-
-                                   if (needsPopModal)
-                                       Main.popModal(this._group, timestamp);
                                })
                          });
     },
 
+    // Drop modal status without closing the dialog; this makes the
+    // dialog insensitive as well, so it needs to be followed shortly
+    // by either a close() or a pushModal()
+    popModal: function(timestamp) {
+        if (!this._hasModal)
+            return;
+
+        let focus = global.stage.key_focus;
+        if (focus && this._group.contains(focus))
+            this._savedKeyFocus = focus;
+        else
+            this._savedKeyFocus = null;
+        Main.popModal(this._group, timestamp);
+        global.gdk_screen.get_display().sync();
+        this._hasModal = false;
+
+        this._eventBlocker.raise_top();
+    },
+
+    pushModal: function (timestamp) {
+        if (this._hasModal)
+            return true;
+        if (!Main.pushModal(this._group, timestamp))
+            return false;
+
+        this._hasModal = true;
+        if (this._savedKeyFocus) {
+            this._savedKeyFocus.grab_key_focus();
+            this._savedKeyFocus = null;
+        } else
+            this._initialKeyFocus.grab_key_focus();
+
+        this._eventBlocker.lower_bottom();
+        return true;
+    },
+
     // This method is like close, but fades the dialog out much slower,
     // and leaves the lightbox in place. Once in the faded out state,
     // the dialog can be brought back by an open call, or the lightbox
@@ -220,6 +261,7 @@ ModalDialog.prototype = {
         if (this.state == State.FADED_OUT)
             return;
 
+        this.popModal(timestamp);
         Tweener.addTween(this._dialogLayout,
                          { opacity: 0,
                            time:    FADE_OUT_DIALOG_TIME,
@@ -227,7 +269,6 @@ ModalDialog.prototype = {
                            onComplete: Lang.bind(this,
                                function() {
                                    this.state = State.FADED_OUT;
-                                   Main.popModal(this._group, timestamp);
                                })
                          });
     }
