@@ -9,11 +9,10 @@ const Pango = imports.gi.Pango;
 const Shell = imports.gi.Shell;
 const St = imports.gi.St;
 const Signals = imports.signals;
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
 
 const Config = imports.misc.config;
 const CtrlAltTab = imports.ui.ctrlAltTab;
+const Layout = imports.ui.layout;
 const Overview = imports.ui.overview;
 const PopupMenu = imports.ui.popupMenu;
 const PanelMenu = imports.ui.panelMenu;
@@ -26,8 +25,6 @@ const PANEL_ICON_SIZE = 24;
 
 const STARTUP_ANIMATION_TIME = 0.2;
 
-const HOT_CORNER_ACTIVATION_TIMEOUT = 0.5;
-
 const BUTTON_DND_ACTIVATION_TIMEOUT = 250;
 
 const ANIMATED_ICON_UPDATE_TIMEOUT = 100;
@@ -260,10 +257,14 @@ AppMenuButton.prototype = {
         this._iconBox = new Shell.Slicer({ name: 'appMenuIcon' });
         this._iconBox.connect('style-changed',
                               Lang.bind(this, this._onIconBoxStyleChanged));
+        this._iconBox.connect('notify::allocation',
+                              Lang.bind(this, this._updateIconBoxClip));
         this._container.add_actor(this._iconBox);
         this._label = new TextShadower();
         this._container.add_actor(this._label.actor);
 
+        this._iconBottomClip = 0;
+
         this._quitMenu = new PopupMenu.PopupMenuItem('');
         this.menu.addMenuItem(this._quitMenu);
         this._quitMenu.connect('activate', Lang.bind(this, this._onQuit));
@@ -300,6 +301,7 @@ AppMenuButton.prototype = {
 
         this._visible = true;
         this.actor.show();
+        this.actor.reactive = true;
 
         if (!this._targetIsCurrent)
             return;
@@ -316,6 +318,7 @@ AppMenuButton.prototype = {
             return;
 
         this._visible = false;
+        this.actor.reactive = false;
         if (!this._targetIsCurrent) {
             this.actor.hide();
             return;
@@ -334,11 +337,16 @@ AppMenuButton.prototype = {
 
     _onIconBoxStyleChanged: function() {
         let node = this._iconBox.get_theme_node();
-        let bottomClip = node.get_length('app-icon-bottom-clip');
-        if (bottomClip > 0)
+        this._iconBottomClip = node.get_length('app-icon-bottom-clip');
+        this._updateIconBoxClip();
+    },
+
+    _updateIconBoxClip: function() {
+        let allocation = this._iconBox.allocation;
+        if (this._iconBottomClip > 0)
             this._iconBox.set_clip(0, 0,
-                                   this._iconBox.width,
-                                   this._iconBox.height - bottomClip);
+                                   allocation.x2 - allocation.x1,
+                                   allocation.y2 - allocation.y1 - this._iconBottomClip);
         else
             this._iconBox.remove_clip();
     },
@@ -535,13 +543,163 @@ AppMenuButton.prototype = {
 
 Signals.addSignalMethods(AppMenuButton.prototype);
 
+// Activities button. Because everything else in the top bar is a
+// PanelMenu.Button, it simplifies some things to make this be one too.
+// We just hack it up to not actually have a menu attached to it.
+function ActivitiesButton() {
+    this._init.apply(this, arguments);
+}
 
-function PanelCorner(side) {
-    this._init(side);
+ActivitiesButton.prototype = {
+    __proto__: PanelMenu.Button.prototype,
+
+    _init: function() {
+        PanelMenu.Button.prototype._init.call(this, 0.0);
+
+        let container = new Shell.GenericContainer();
+        container.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
+        container.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
+        container.connect('allocate', Lang.bind(this, this._allocate));
+        this.actor.child = container;
+        this.actor.name = 'panelActivities';
+
+        /* Translators: If there is no suitable word for "Activities"
+           in your language, you can use the word for "Overview". */
+        this._label = new St.Label({ text: _("Activities") });
+        container.add_actor(this._label);
+
+        this._hotCorner = new Layout.HotCorner();
+        container.add_actor(this._hotCorner.actor);
+
+        // Hack up our menu...
+        this.menu.open = Lang.bind(this, this._onMenuOpenRequest);
+        this.menu.close = Lang.bind(this, this._onMenuCloseRequest);
+        this.menu.toggle = Lang.bind(this, this._onMenuToggleRequest);
+
+        this.actor.connect('captured-event', Lang.bind(this, this._onCapturedEvent));
+        this.actor.connect_after('button-release-event', Lang.bind(this, this._onButtonRelease));
+        this.actor.connect_after('key-release-event', Lang.bind(this, this._onKeyRelease));
+
+        Main.overview.connect('showing', Lang.bind(this, function() {
+            this.actor.add_style_pseudo_class('overview');
+            this._escapeMenuGrab();
+        }));
+        Main.overview.connect('hiding', Lang.bind(this, function() {
+            this.actor.remove_style_pseudo_class('overview');
+            this._escapeMenuGrab();
+        }));
+
+        this._xdndTimeOut = 0;
+    },
+
+    _getPreferredWidth: function(actor, forHeight, alloc) {
+        [alloc.min_size, alloc.natural_size] = this._label.get_preferred_width(forHeight);
+    },
+
+    _getPreferredHeight: function(actor, forWidth, alloc) {
+        [alloc.min_size, alloc.natural_size] = this._label.get_preferred_height(forWidth);
+    },
+
+    _allocate: function(actor, box, flags) {
+        this._label.allocate(box, flags);
+
+        // The hot corner needs to be outside any padding/alignment
+        // that has been imposed on us
+        let primary = Main.layoutManager.primaryMonitor;
+        let hotBox = new Clutter.ActorBox();
+        let ok, x, y;
+        if (actor.get_direction() == St.TextDirection.LTR) {
+            [ok, x, y] = actor.transform_stage_point(primary.x, primary.y)
+        } else {
+            [ok, x, y] = actor.transform_stage_point(primary.x + primary.width, primary.y);
+            // hotCorner.actor has northeast gravity, so we don't need
+            // to adjust x for its width
+        }
+
+        hotBox.x1 = Math.round(x);
+        hotBox.x2 = hotBox.x1 + this._hotCorner.actor.width;
+        hotBox.y1 = Math.round(y);
+        hotBox.y2 = hotBox.y1 + this._hotCorner.actor.height;
+        this._hotCorner.actor.allocate(hotBox, flags);
+    },
+
+    handleDragOver: function(source, actor, x, y, time) {
+        if (source != Main.xdndHandler)
+            return;
+
+        if (this._xdndTimeOut != 0)
+            Mainloop.source_remove(this._xdndTimeOut);
+        this._xdndTimeOut = Mainloop.timeout_add(BUTTON_DND_ACTIVATION_TIMEOUT,
+                                                 Lang.bind(this, this._xdndShowOverview, actor));
+    },
+
+    _escapeMenuGrab: function() {
+        if (this.menu.isOpen)
+            this.menu.close();
+    },
+
+    _onCapturedEvent: function(actor, event) {
+        if (event.type() == Clutter.EventType.BUTTON_PRESS) {
+            if (!this._hotCorner.shouldToggleOverviewOnClick())
+                return true;
+        }
+        return false;
+    },
+
+    _onMenuOpenRequest: function() {
+        this.menu.isOpen = true;
+        this.menu.emit('open-state-changed', true);
+    },
+
+    _onMenuCloseRequest: function() {
+        this.menu.isOpen = false;
+        this.menu.emit('open-state-changed', false);
+    },
+
+    _onMenuToggleRequest: function() {
+        this.menu.isOpen = !this.menu.isOpen;
+        this.menu.emit('open-state-changed', this.menu.isOpen);
+    },
+
+    _onButtonRelease: function() {
+        if (this.menu.isOpen) {
+            this.menu.close();
+            Main.overview.toggle();
+        }
+    },
+
+    _onKeyRelease: function(actor, event) {
+        let symbol = event.get_key_symbol();
+        if (symbol == Clutter.KEY_Return || symbol == Clutter.KEY_space) {
+            if (this.menu.isOpen)
+                this.menu.close();
+            Main.overview.toggle();
+        }
+    },
+
+    _xdndShowOverview: function(actor) {
+        let [x, y, mask] = global.get_pointer();
+        let pickedActor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
+
+        if (pickedActor == this.actor) {
+            if (!Main.overview.visible && !Main.overview.animationInProgress) {
+                Main.overview.showTemporarily();
+                Main.overview.beginItemDrag(actor);
+            }
+        }
+
+        Mainloop.source_remove(this._xdndTimeOut);
+        this._xdndTimeOut = 0;
+    }
+};
+
+function PanelCorner(panel, side) {
+    this._init(panel, side);
 }
 
 PanelCorner.prototype = {
-    _init: function(side) {
+    _init: function(panel, side) {
+        this._panel = panel;
         this._side = side;
         this.actor = new St.DrawingArea({ style_class: 'panel-corner' });
         this.actor.connect('repaint', Lang.bind(this, this._repaint));
@@ -617,185 +775,14 @@ PanelCorner.prototype = {
         this.actor.set_size(cornerRadius,
                             innerBorderWidth + cornerRadius);
         if (this._side == St.Side.LEFT)
-            this.actor.set_position(Main.panel.actor.x,
-                                    Main.panel.actor.y + Main.panel.actor.height - innerBorderWidth);
+            this.actor.set_position(this._panel.actor.x,
+                                    this._panel.actor.y + this._panel.actor.height - innerBorderWidth);
         else
-            this.actor.set_position(Main.panel.actor.x + Main.panel.actor.width - cornerRadius,
-                                    Main.panel.actor.y + Main.panel.actor.height - innerBorderWidth);
+            this.actor.set_position(this._panel.actor.x + this._panel.actor.width - cornerRadius,
+                                    this._panel.actor.y + this._panel.actor.height - innerBorderWidth);
     }
 };
 
-/**
- * HotCorner:
- *
- * This class manages the "hot corner" that can toggle switching to
- * overview.
- */
-function HotCorner(button) {
-    this._init(button);
-}
-
-HotCorner.prototype = {
-    _init : function(button) {
-        // This is the activities button associated with this hot corner,
-        // if this is on the primary monitor (or null with the corner is
-        // on a different monitor)
-        this._button = button;
-
-        // We use this flag to mark the case where the user has entered the
-        // hot corner and has not left both the hot corner and a surrounding
-        // guard area (the "environs"). This avoids triggering the hot corner
-        // multiple times due to an accidental jitter.
-        this._entered = false;
-
-        this.actor = new Clutter.Group({ width: 3,
-                                         height: 3,
-                                         reactive: true });
-
-        this._corner = new Clutter.Rectangle({ width: 1,
-                                               height: 1,
-                                               opacity: 0,
-                                               reactive: true });
-
-        this.actor.add_actor(this._corner);
-
-        if (St.Widget.get_default_direction() == St.TextDirection.RTL) {
-            this._corner.set_position(this.actor.width - this._corner.width, 0);
-            this.actor.set_anchor_point_from_gravity(Clutter.Gravity.NORTH_EAST);
-        } else {
-            this._corner.set_position(0, 0);
-        }
-
-        this._activationTime = 0;
-
-        this.actor.connect('enter-event',
-                           Lang.bind(this, this._onEnvironsEntered));
-        this.actor.connect('leave-event',
-                           Lang.bind(this, this._onEnvironsLeft));
-        // Clicking on the hot corner environs should result in the same bahavior
-        // as clicking on the hot corner.
-        this.actor.connect('button-release-event',
-                           Lang.bind(this, this._onCornerClicked));
-
-        // In addition to being triggered by the mouse enter event, the hot corner
-        // can be triggered by clicking on it. This is useful if the user wants to
-        // undo the effect of triggering the hot corner once in the hot corner.
-        this._corner.connect('enter-event',
-                             Lang.bind(this, this._onCornerEntered));
-        this._corner.connect('button-release-event',
-                             Lang.bind(this, this._onCornerClicked));
-        this._corner.connect('leave-event',
-                             Lang.bind(this, this._onCornerLeft));
-
-        this._corner._delegate = this._corner;
-        this._corner.handleDragOver = Lang.bind(this,
-            function(source, actor, x, y, time) {
-                 if (source == Main.xdndHandler) {
-                    if(!Main.overview.visible && !Main.overview.animationInProgress) {
-                        this.rippleAnimation();
-                        Main.overview.showTemporarily();
-                        Main.overview.beginItemDrag(actor);
-                    }
-                 }
-            });
-
-        Main.chrome.addActor(this.actor, { visibleInOverview: true, affectsStruts: false });
-    },
-
-    destroy: function() {
-        this.actor.destroy();
-    },
-
-    _addRipple : function(delay, time, startScale, startOpacity, finalScale, finalOpacity) {
-        // We draw a ripple by using a source image and animating it scaling
-        // outwards and fading away. We want the ripples to move linearly
-        // or it looks unrealistic, but if the opacity of the ripple goes
-        // linearly to zero it fades away too quickly, so we use Tweener's
-        // 'onUpdate' to give a non-linear curve to the fade-away and make
-        // it more visible in the middle section.
-
-        let [x, y] = this._corner.get_transformed_position();
-        let ripple = new St.BoxLayout({ style_class: 'ripple-box',
-                                        opacity: 255 * Math.sqrt(startOpacity),
-                                        scale_x: startScale,
-                                        scale_y: startScale,
-                                        x: x,
-                                        y: y });
-        ripple._opacity =  startOpacity;
-        if (ripple.get_direction() == St.TextDirection.RTL)
-            ripple.set_anchor_point_from_gravity(Clutter.Gravity.NORTH_EAST);
-        Tweener.addTween(ripple, { _opacity: finalOpacity,
-                                   scale_x: finalScale,
-                                   scale_y: finalScale,
-                                   delay: delay,
-                                   time: time,
-                                   transition: 'linear',
-                                   onUpdate: function() { ripple.opacity = 255 * Math.sqrt(ripple._opacity); },
-                                   onComplete: function() { ripple.destroy(); } });
-        Main.uiGroup.add_actor(ripple);
-    },
-
-    rippleAnimation: function() {
-        // Show three concentric ripples expanding outwards; the exact
-        // parameters were found by trial and error, so don't look
-        // for them to make perfect sense mathematically
-
-        //              delay  time  scale opacity => scale opacity
-        this._addRipple(0.0,   0.83,  0.25,  1.0,    1.5,  0.0);
-        this._addRipple(0.05,  1.0,   0.0,   0.7,    1.25, 0.0);
-        this._addRipple(0.35,  1.0,   0.0,   0.3,    1,    0.0);
-    },
-
-    _onEnvironsEntered : function() {
-        if (this._button)
-            this._button.hover = true;
-    },
-
-    _onCornerEntered : function() {
-        if (!this._entered) {
-            this._entered = true;
-            if (!Main.overview.animationInProgress) {
-                this._activationTime = Date.now() / 1000;
-
-                this.rippleAnimation();
-                Main.overview.toggle();
-            }
-        }
-        return false;
-    },
-
-    _onCornerClicked : function() {
-         if (!Main.overview.animationInProgress)
-             this.maybeToggleOverviewOnClick();
-         return false;
-    },
-
-    _onCornerLeft : function(actor, event) {
-        if (event.get_related() != this.actor)
-            this._entered = false;
-        // Consume event, otherwise this will confuse onEnvironsLeft
-        return true;
-    },
-
-    _onEnvironsLeft : function(actor, event) {
-        if (this._button)
-            this._button.hover = false;
-
-        if (event.get_related() != this._corner)
-            this._entered = false;
-        return false;
-    },
-
-    // Toggles the overview unless this is the first click on the Activities button within the HOT_CORNER_ACTIVATION_TIMEOUT time
-    // of the hot corner being triggered. This check avoids opening and closing the overview if the user both triggered the hot corner
-    // and clicked the Activities button.
-    maybeToggleOverviewOnClick: function() {
-        if (this._activationTime == 0 || Date.now() / 1000 - this._activationTime > HOT_CORNER_ACTIVATION_TIMEOUT)
-            Main.overview.toggle();
-        this._activationTime = 0;
-    }
-}
-
 
 function Panel() {
     this._init();
@@ -808,6 +795,8 @@ Panel.prototype = {
                                         reactive: true });
         this.actor._delegate = this;
 
+        this._statusArea = {};
+
         Main.overview.connect('shown', Lang.bind(this, function () {
             this.actor.add_style_class_name('in-overview');
         }));
@@ -823,8 +812,8 @@ Panel.prototype = {
         this._centerBox = new St.BoxLayout({ name: 'panelCenter' });
         this._rightBox = new St.BoxLayout({ name: 'panelRight' });
 
-        this._leftCorner = new PanelCorner(St.Side.LEFT);
-        this._rightCorner = new PanelCorner(St.Side.RIGHT);
+        this._leftCorner = new PanelCorner(this, St.Side.LEFT);
+        this._rightCorner = new PanelCorner(this, St.Side.RIGHT);
 
         /* This box container ensures that the centerBox is positioned in the *absolute*
          * center, but can be pushed aside if necessary. */
@@ -898,31 +887,16 @@ Panel.prototype = {
         }));
 
         /* Button on the left side of the panel. */
-        /* Translators: If there is no suitable word for "Activities" in your language, you can use the word for "Overview". */
-        let label = new St.Label({ text: _("Activities") });
-        this.button = new St.Button({ name: 'panelActivities',
-                                      style_class: 'panel-button',
-                                      reactive: true,
-                                      can_focus: true });
-        this.button.set_child(label);
-        this.button._delegate = this.button;
-        this.button._xdndTimeOut = 0;
-        this.button.handleDragOver = Lang.bind(this,
-            function(source, actor, x, y, time) {
-                 if (source == Main.xdndHandler) {
-                    if (this.button._xdndTimeOut != 0)
-                        Mainloop.source_remove(this.button._xdndTimeOut);
-                    this.button._xdndTimeOut = Mainloop.timeout_add(BUTTON_DND_ACTIVATION_TIMEOUT,
-                                                                    Lang.bind(this,
-                                                                                function() {
-                                                                                    this._xdndShowOverview(actor);
-                                                                                }));
-                 }
-            });
-        this._leftBox.add(this.button);
+        this._activitiesButton = new ActivitiesButton();
+        this._activities = this._activitiesButton.actor;
+        this._leftBox.add(this._activities);
 
-        // Synchronize the buttons pseudo classes with its corner
-        this.button.connect('style-changed', Lang.bind(this,
+        // The activities button has a pretend menu, so as to integrate
+        // more cleanly with the rest of the panel
+        this._menus.addMenu(this._activitiesButton.menu);
+
+        // Synchronize the button's pseudo classes with its corner
+        this._activities.connect('style-changed', Lang.bind(this,
             function(actor) {
                 let rtl = actor.get_direction() == St.TextDirection.RTL;
                 let corner = rtl ? this._rightCorner : this._leftCorner;
@@ -930,12 +904,10 @@ Panel.prototype = {
                 corner.actor.set_style_pseudo_class(pseudoClass);
             }));
 
-        this._hotCorner = null;
+        this._appMenu = new AppMenuButton();
+        this._leftBox.add(this._appMenu.actor);
 
-        let appMenuButton = new AppMenuButton();
-        this._leftBox.add(appMenuButton.actor);
-
-        this._menus.addMenu(appMenuButton.menu);
+        this._menus.addMenu(this._appMenu.menu);
 
         /* center */
         this._dateMenu = new DateMenu.DateMenuButton();
@@ -954,12 +926,12 @@ Panel.prototype = {
         this._rightBox.add(this._trayBox);
         this._rightBox.add(this._statusBox);
 
-        this._statusmenu = new StatusMenu.StatusMenuButton();
-        this._statusmenu.actor.name = 'panelStatus';
-        this._rightBox.add(this._statusmenu.actor);
+        this._userMenu = new StatusMenu.StatusMenuButton();
+        this._userMenu.actor.name = 'panelStatus';
+        this._rightBox.add(this._userMenu.actor);
 
         // Synchronize the buttons pseudo classes with its corner
-        this._statusmenu.actor.connect('style-changed', Lang.bind(this,
+        this._userMenu.actor.connect('style-changed', Lang.bind(this,
             function(actor) {
                 let rtl = actor.get_direction() == St.TextDirection.RTL;
                 let corner = rtl ? this._leftCorner : this._rightCorner;
@@ -970,65 +942,17 @@ Panel.prototype = {
         Main.statusIconDispatcher.connect('status-icon-added', Lang.bind(this, this._onTrayIconAdded));
         Main.statusIconDispatcher.connect('status-icon-removed', Lang.bind(this, this._onTrayIconRemoved));
 
-        // TODO: decide what to do with the rest of the panel in the Overview mode (make it fade-out, become non-reactive, etc.)
-        // We get into the Overview mode on button-press-event as opposed to button-release-event because eventually we'll probably
-        // have the Overview act like a menu that allows the user to release the mouse on the activity the user wants
-        // to switch to.
-        this.button.connect('clicked', Lang.bind(this, function(b) {
-            if (!Main.overview.animationInProgress) {
-                this._hotCorner.maybeToggleOverviewOnClick();
-                return true;
-            } else {
-                return false;
-            }
-        }));
-        // In addition to pressing the button, the Overview can be entered and exited by other means, such as
-        // pressing the System key, Alt+F1 or Esc. We want the button to be pressed in when the Overview is entered
-        // and to be released when it is exited regardless of how it was triggered.
-        Main.overview.connect('showing', Lang.bind(this, function() {
-            this.button.checked = true;
-        }));
-        Main.overview.connect('hiding', Lang.bind(this, function() {
-            this.button.checked = false;
-        }));
-
-        Main.chrome.addActor(this.actor, { visibleInOverview: true });
-        Main.chrome.addActor(this._leftCorner.actor, { visibleInOverview: true,
-                                                 affectsStruts: false,
-                                                 affectsInputRegion: false });
-        Main.chrome.addActor(this._rightCorner.actor, { visibleInOverview: true,
-                                                  affectsStruts: false,
-                                                  affectsInputRegion: false });
+        Main.chrome.addActor(this.actor);
+        Main.chrome.addActor(this._leftCorner.actor, { affectsStruts: false,
+                                                       affectsInputRegion: false });
+        Main.chrome.addActor(this._rightCorner.actor, { affectsStruts: false,
+                                                        affectsInputRegion: false });
 
         Main.ctrlAltTabManager.addGroup(this.actor, _("Top Bar"), 'start-here',
                                         { sortGroup: CtrlAltTab.SortGroup.TOP });
-    },
 
-    _xdndShowOverview: function (actor) {
-        let [x, y, mask] = global.get_pointer();
-        let pickedActor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
-
-        if (pickedActor != this.button) {
-            Mainloop.source_remove(this.button._xdndTimeOut);
-            this.button._xdndTimeOut = 0;
-            return;
-        }
-
-        if(!Main.overview.visible && !Main.overview.animationInProgress) {
-            Main.overview.showTemporarily();
-            Main.overview.beginItemDrag(actor);
-        }
-
-        Mainloop.source_remove(this.button._xdndTimeOut);
-        this.button._xdndTimeOut = 0;
-    },
-
-
-    // While there can be multiple hotcorners (one per monitor), the hot corner
-    // that is on top of the Activities button is special since it needs special
-    // coordination with clicking on that button
-    setHotCorner: function(corner) {
-        this._hotCorner = corner;
+        Main.layoutManager.connect('monitors-changed', Lang.bind(this, this._relayout));
+        this._relayout();
     },
 
     startStatusArea: function() {
@@ -1042,11 +966,13 @@ Panel.prototype = {
             let indicator = new constructor();
             this._statusBox.add(indicator.actor);
             this._menus.addMenu(indicator.menu);
+
+            this._statusArea[role] = indicator;
         }
 
         // PopupMenuManager depends on menus being added in order for
         // keyboard navigation
-        this._menus.addMenu(this._statusmenu.menu);
+        this._menus.addMenu(this._userMenu.menu);
     },
 
     startupAnimation: function() {
@@ -1073,8 +999,8 @@ Panel.prototype = {
                          });
     },
 
-    relayout: function() {
-        let primary = global.get_primary_monitor();
+    _relayout: function() {
+        let primary = Main.layoutManager.primaryMonitor;
 
         this.actor.set_position(primary.x, primary.y);
         this.actor.set_size(primary.width, -1);
