@@ -35,7 +35,7 @@ Button.prototype = {
             // Setting the max-height won't do any good if the minimum height of the
             // menu is higher then the screen; it's useful if part of the menu is
             // scrollable so the minimum height is smaller than the natural height
-            let monitor = global.get_primary_monitor();
+            let monitor = Main.layoutManager.primaryMonitor;
             this.menu.actor.style = ('max-height: ' +
                                      Math.round(monitor.height - Main.panel.actor.height) +
                                      'px;');
