@@ -711,7 +711,6 @@ PopupSwitchMenuItem.prototype = {
         this._statusLabel = new St.Label({ text: '',
                                            style_class: 'popup-inactive-menu-item'
                                          });
-        this._switch = new Switch(false);
         this._statusBin.child = this._switch.actor;
     },
 
@@ -947,6 +946,14 @@ PopupMenuBase.prototype = {
         });
     },
 
+    get firstMenuItem() {
+        let items = this._getMenuItems();
+        if (items.length)
+            return items[0];
+        else
+            return null;
+    },
+
     removeAll: function() {
         let children = this._getMenuItems();
         for (let i = 0; i < children.length; i++) {
