@@ -10,13 +10,11 @@ const Shell = imports.gi.Shell;
 const Signals = imports.signals;
 const St = imports.gi.St;
 
+const Main = imports.ui.main;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
 const Util = imports.misc.util;
 
-const Gettext = imports.gettext.domain('gnome-shell');
-const _ = Gettext.gettext;
-
 const A11Y_SCHEMA = 'org.gnome.desktop.a11y.keyboard';
 const KEY_STICKY_KEYS_ENABLED = 'stickykeys-enable';
 const KEY_BOUNCE_KEYS_ENABLED = 'bouncekeys-enable';
@@ -66,13 +64,13 @@ ATIndicator.prototype = {
         let textZoom = this._buildFontItem();
         this.menu.addMenuItem(textZoom);
 
-        let screenReader = this._buildItem(_("Screen Reader"), APPLICATIONS_SCHEMA,
-                                                               'screen-reader-enabled');
-        this.menu.addMenuItem(screenReader);
+//        let screenReader = this._buildItem(_("Screen Reader"), APPLICATIONS_SCHEMA,
+//                                                               'screen-reader-enabled');
+//        this.menu.addMenuItem(screenReader);
 
-        let screenKeyboard = this._buildItem(_("Screen Keyboard"), APPLICATIONS_SCHEMA,
-                                                                   'screen-keyboard-enabled');
-        this.menu.addMenuItem(screenKeyboard);
+//        let screenKeyboard = this._buildItem(_("Screen Keyboard"), APPLICATIONS_SCHEMA,
+//                                                                   'screen-keyboard-enabled');
+//        this.menu.addMenuItem(screenKeyboard);
 
         let visualBell = this._buildItemGConf(_("Visual Alerts"), client, KEY_VISUAL_BELL);
         this.menu.addMenuItem(visualBell);
@@ -91,7 +89,9 @@ ATIndicator.prototype = {
 
         this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
         this.menu.addAction(_("Universal Access Settings"), function() {
-            Util.spawnDesktop('gnome-universal-access-panel');
+            Main.overview.hide();
+            let app = Shell.AppSystem.get_default().get_app('gnome-universal-access-panel.desktop');
+            app.activate(-1);
         });
     },
 
