@@ -24,6 +24,9 @@ const SCROLLBACK_IDLE_LENGTH = 5;
 // See Source._displayPendingMessages
 const SCROLLBACK_HISTORY_LINES = 10;
 
+// See Notification._onEntryChanged
+const COMPOSING_STOP_TIMEOUT = 5;
+
 const NotificationDirection = {
     SENT: 'chat-sent',
     RECEIVED: 'chat-received'
@@ -69,8 +72,9 @@ function Client() {
 
 Client.prototype = {
     _init : function() {
-        // channel path -> Source
-        this._sources = {};
+        // channel path -> ChatSource
+        this._chatSources = {};
+        this._chatState = Tp.ChannelChatState.ACTIVE;
 
         // Set up a SimpleObserver, which will call _observeChannels whenever a
         // channel matching its filters is detected.
@@ -87,6 +91,11 @@ Client.prototype = {
         this._tpClient.set_handle_channels_func(
             Lang.bind(this, this._handleChannels));
 
+        // Allow other clients (such as Empathy) to pre-empt our channels if
+        // needed
+        this._tpClient.set_delegated_channels_callback(
+            Lang.bind(this, this._delegatedChannelsCb));
+
         try {
             this._tpClient.register();
         } catch (e) {
@@ -130,20 +139,20 @@ Client.prototype = {
                             return;
 
                         /* We got the TpContact */
-                        this._createSource(account, conn, channel, contacts[0]);
+                        this._createChatSource(account, conn, channel, contacts[0]);
                     }), null);
         }
 
         context.accept();
     },
 
-    _createSource: function(account, conn, channel, contact) {
-        if (this._sources[channel.get_object_path()])
+    _createChatSource: function(account, conn, channel, contact) {
+        if (this._chatSources[channel.get_object_path()])
             return;
 
-        let source = new Source(account, conn, channel, contact, this._tpClient);
+        let source = new ChatSource(account, conn, channel, contact, this._tpClient);
 
-        this._sources[channel.get_object_path()] = source;
+        this._chatSources[channel.get_object_path()] = source;
         source.connect('destroy', Lang.bind(this,
                        function() {
                            if (this._tpClient.is_handling_channel(channel)) {
@@ -154,10 +163,16 @@ Client.prototype = {
                                });
                            }
 
-                           delete this._sources[channel.get_object_path()];
+                           delete this._chatSources[channel.get_object_path()];
                        }));
     },
 
+    _handleChannels: function(handler, account, conn, channels,
+                              requests, user_action_time, context) {
+        this._handlingChannels(account, conn, channels);
+        context.accept();
+    },
+
     _handlingChannels: function(account, conn, channels) {
         let len = channels.length;
         for (let i = 0; i < len; i++) {
@@ -171,40 +186,127 @@ Client.prototype = {
 
             if (this._tpClient.is_handling_channel(channel)) {
                 // We are already handling the channel, display the source
-                let source = this._sources[channel.get_object_path()];
+                let source = this._chatSources[channel.get_object_path()];
                 if (source)
                     source.notify();
             }
         }
     },
 
-    _approveChannels: function(approver, account, conn, channels,
-                               dispatchOp, context) {
-        // Approve the channels right away as we are going to handle it
-        dispatchOp.claim_with_async(this._tpClient,
-                                    Lang.bind (this, function(dispatchOp, result) {
-            try {
-                dispatchOp.claim_with_finish(result);
-                this._handlingChannels(account, conn, channels);
-            } catch (err) {
-                global.logError('Failed to Claim channel: ' + err);
-            }}));
+    _displayRoomInvitation: function(conn, channel, dispatchOp, context) {
+        // We can only approve the rooms if we have been invited to it
+        let selfHandle = channel.group_get_self_handle();
+        if (selfHandle == 0) {
+            Shell.decline_dispatch_op(context, 'Not invited to the room');
+            return;
+        }
 
+        let [invited, inviter, reason, msg] = channel.group_get_local_pending_info(selfHandle);
+        if (!invited) {
+            Shell.decline_dispatch_op(context, 'Not invited to the room');
+            return;
+        }
+
+        // Request a TpContact for the inviter
+        Shell.get_tp_contacts(conn, [inviter],
+                contactFeatures,
+                Lang.bind(this, this._createRoomInviteSource, channel, context, dispatchOp));
+
+        context.delay();
+     },
+
+    _createRoomInviteSource: function(connection, contacts, failed, channel, context, dispatchOp) {
+        if (contacts.length < 1) {
+            Shell.decline_dispatch_op(context, 'Failed to get inviter');
+            return;
+        }
+
+        // We got the TpContact
+
+        // FIXME: We don't have a 'chat room' icon (bgo #653737) use
+        // system-users for now as Empathy does.
+        let source = new ApproverSource(dispatchOp, _("Invitation"), 'system-users');
+        Main.messageTray.add(source);
+
+        let notif = new RoomInviteNotification(source, dispatchOp, channel, contacts[0]);
+        source.notify(notif);
         context.accept();
     },
 
-    _handleChannels: function(handler, account, conn, channels,
-                              requests, user_action_time, context) {
-        this._handlingChannels(account, conn, channels);
+    _approveChannels: function(approver, account, conn, channels,
+                               dispatchOp, context) {
+        let channel = channels[0];
+        let chanType = channel.get_channel_type();
+
+        if (chanType == Tp.IFACE_CHANNEL_TYPE_TEXT)
+            this._approveTextChannel(account, conn, channel, dispatchOp, context);
+        else if (chanType == Tp.IFACE_CHANNEL_TYPE_STREAMED_MEDIA ||
+                 chanType == 'org.freedesktop.Telepathy.Channel.Type.Call.DRAFT')
+            this._approveCall(account, conn, channel, dispatchOp, context);
+    },
+
+    _approveTextChannel: function(account, conn, channel, dispatchOp, context) {
+        let [targetHandle, targetHandleType] = channel.get_handle();
+
+        if (targetHandleType == Tp.HandleType.CONTACT) {
+            // Approve private text channels right away as we are going to handle it
+            dispatchOp.claim_with_async(this._tpClient,
+                                        Lang.bind(this, function(dispatchOp, result) {
+                try {
+                    dispatchOp.claim_with_finish(result);
+                    this._handlingChannels(account, conn, [channel]);
+                } catch (err) {
+                    throw new Error('Failed to Claim channel: ' + err);
+                }}));
+
+            context.accept();
+        } else {
+            this._displayRoomInvitation(conn, channel, dispatchOp, context);
+        }
+    },
+
+    _approveCall: function(account, conn, channel, dispatchOp, context) {
+        let [targetHandle, targetHandleType] = channel.get_handle();
+
+        Shell.get_tp_contacts(conn, [targetHandle],
+                contactFeatures,
+                Lang.bind(this, this._createAudioVideoSource, channel, context, dispatchOp));
+    },
+
+    _createAudioVideoSource: function(connection, contacts, failed, channel, context, dispatchOp) {
+        if (contacts.length < 1) {
+            Shell.decline_dispatch_op(context, 'Failed to get inviter');
+            return;
+        }
+
+        let isVideo = false;
+
+        let props = channel.borrow_immutable_properties();
+
+        if (props['org.freedesktop.Telepathy.Channel.Type.Call.DRAFT.InitialVideo'] ||
+            props[Tp.PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO])
+          isVideo = true;
+
+        // We got the TpContact
+        let source = new ApproverSource(dispatchOp, _("Call"), isVideo ? 'camera-web' : 'audio-input-microphone');
+        Main.messageTray.add(source);
+
+        let notif = new AudioVideoNotification(source, dispatchOp, channel, contacts[0], isVideo);
+        source.notify(notif);
         context.accept();
+    },
+
+    _delegatedChannelsCb: function(client, channels) {
+        // Nothing to do as we don't make a distinction between observed and
+        // handled channels.
     }
 };
 
-function Source(account, conn, channel, contact, client) {
+function ChatSource(account, conn, channel, contact, client) {
     this._init(account, conn, channel, contact, client);
 }
 
-Source.prototype = {
+ChatSource.prototype = {
     __proto__:  MessageTray.Source.prototype,
 
     _init: function(account, conn, channel, contact, client) {
@@ -222,7 +324,7 @@ Source.prototype = {
         this._channel = channel;
         this._closedId = this._channel.connect('invalidated', Lang.bind(this, this._channelClosed));
 
-        this._notification = new Notification(this);
+        this._notification = new ChatNotification(this);
         this._notification.setUrgency(MessageTray.Urgency.HIGH);
 
         // We ack messages when the message box is collapsed if user has
@@ -326,6 +428,8 @@ Source.prototype = {
             this._pendingMessages.push(message);
         }
 
+        this._updateCount();
+
         let showTimestamp = false;
 
         for (let i = 0; i < logMessages.length; i++) {
@@ -370,11 +474,16 @@ Source.prototype = {
         this.destroy();
     },
 
+    _updateCount: function() {
+        this._setCount(this._pendingMessages.length, this._pendingMessages.length > 0);
+    },
+
     _messageReceived: function(channel, message) {
         if (message.get_message_type() == Tp.ChannelTextMessageType.DELIVERY_REPORT)
             return;
 
         this._pendingMessages.push(message);
+        this._updateCount();
 
         message = makeMessageFromTpMessage(message, NotificationDirection.RECEIVED);
         this._notification.appendMessage(message);
@@ -407,6 +516,19 @@ Source.prototype = {
         }));
     },
 
+    setChatState: function(state) {
+        // We don't want to send COMPOSING every time a letter is typed into
+        // the entry. We send the state only when it changes. Telepathy/Empathy
+        // might change it behind our back if the user is using both
+        // gnome-shell's entry and the Empathy conversation window. We could
+        // keep track of it with the ChatStateChanged signal but it is good
+        // enough right now.
+        if (state != this._chatState) {
+          this._chatState = state;
+          this._channel.set_chat_state_async(state, null);
+        }
+    },
+
     _presenceChanged: function (contact, presence, status, message) {
         let msg, shouldNotify, title;
 
@@ -445,8 +567,10 @@ Source.prototype = {
     _pendingRemoved: function(channel, message) {
         let idx = this._pendingMessages.indexOf(message);
 
-        if (idx >= 0)
+        if (idx >= 0) {
             this._pendingMessages.splice(idx, 1);
+            this._updateCount();
+        }
         else
             throw new Error('Message not in our pending list: ' + message);
     },
@@ -480,11 +604,11 @@ Source.prototype = {
     }
 };
 
-function Notification(source) {
+function ChatNotification(source) {
     this._init(source);
 }
 
-Notification.prototype = {
+ChatNotification.prototype = {
     __proto__:  MessageTray.Notification.prototype,
 
     _init: function(source) {
@@ -494,6 +618,7 @@ Notification.prototype = {
         this._responseEntry = new St.Entry({ style_class: 'chat-response',
                                              can_focus: true });
         this._responseEntry.clutter_text.connect('activate', Lang.bind(this, this._onEntryActivated));
+        this._responseEntry.clutter_text.connect('text-changed', Lang.bind(this, this._onEntryChanged));
         this.setActionArea(this._responseEntry);
 
         this._oldMaxScrollAdjustment = 0;
@@ -510,6 +635,7 @@ Notification.prototype = {
 
         this._history = [];
         this._timestampTimeoutId = 0;
+        this._composingTimeoutId = 0;
     },
 
     /**
@@ -682,5 +808,166 @@ Notification.prototype = {
         // see Source._messageSent
         this._responseEntry.set_text('');
         this.source.respond(text);
+    },
+
+    _composingStopTimeout: function() {
+        this._composingTimeoutId = 0;
+
+        this.source.setChatState(Tp.ChannelChatState.PAUSED);
+
+        return false;
+    },
+
+    _onEntryChanged: function() {
+        let text = this._responseEntry.get_text();
+
+        // If we're typing, we want to send COMPOSING.
+        // If we empty the entry, we want to send ACTIVE.
+        // If we've stopped typing for COMPOSING_STOP_TIMEOUT
+        //    seconds, we want to send PAUSED.
+
+        // Remove composing timeout.
+        if (this._composingTimeoutId > 0) {
+            Mainloop.source_remove(this._composingTimeoutId);
+            this._composingTimeoutId = 0;
+        }
+
+        if (text != '') {
+            this.source.setChatState(Tp.ChannelChatState.COMPOSING);
+
+            this._composingTimeoutId = Mainloop.timeout_add_seconds(
+                COMPOSING_STOP_TIMEOUT,
+                Lang.bind(this, this._composingStopTimeout));
+        } else {
+            this.source.setChatState(Tp.ChannelChatState.ACTIVE);
+        }
+    }
+};
+
+function ApproverSource(dispatchOp, text, icon) {
+    this._init(dispatchOp, text, icon);
+}
+
+ApproverSource.prototype = {
+    __proto__: MessageTray.Source.prototype,
+
+    _init: function(dispatchOp, text, icon) {
+        MessageTray.Source.prototype._init.call(this, text);
+
+        this._icon = icon;
+        this._setSummaryIcon(this.createNotificationIcon());
+
+        this._dispatchOp = dispatchOp;
+
+        // Destroy the source if the channel dispatch operation is invalidated
+        // as we can't approve any more.
+        this._invalidId = dispatchOp.connect('invalidated',
+                                             Lang.bind(this, function(domain, code, msg) {
+            this.destroy();
+        }));
+    },
+
+    destroy: function() {
+        if (this._invalidId != 0) {
+            this._dispatchOp.disconnect(this._invalidId);
+            this._invalidId = 0;
+        }
+
+        MessageTray.Source.prototype.destroy.call(this);
+    },
+
+    createNotificationIcon: function() {
+        return new St.Icon({ icon_name: this._icon,
+                             icon_type: St.IconType.FULLCOLOR,
+                             icon_size: this.ICON_SIZE });
+    }
+}
+
+function RoomInviteNotification(source, dispatchOp, channel, inviter) {
+    this._init(source, dispatchOp, channel, inviter);
+}
+
+RoomInviteNotification.prototype = {
+    __proto__: MessageTray.Notification.prototype,
+
+    _init: function(source, dispatchOp, channel, inviter) {
+        MessageTray.Notification.prototype._init.call(this,
+                                                      source,
+                                                      /* translators: argument is a room name like
+                                                       * room@jabber.org for example. */
+                                                      _("Invitation to %s").format(channel.get_identifier()),
+                                                      null,
+                                                      { customContent: true });
+        this.setResident(true);
+
+        /* translators: first argument is the name of a contact and the second
+         * one the name of a room. "Alice is inviting you to join room@jabber.org
+         * for example. */
+        this.addBody(_("%s is inviting you to join %s").format(inviter.get_alias(), channel.get_identifier()));
+
+        this.addButton('decline', _("Decline"));
+        this.addButton('accept', _("Accept"));
+
+        this.connect('action-invoked', Lang.bind(this, function(self, action) {
+            switch (action) {
+            case 'decline':
+                dispatchOp.leave_channels_async(Tp.ChannelGroupChangeReason.NONE,
+                                                '', function(src, result) {
+                    src.leave_channels_finish(result)});
+                break;
+            case 'accept':
+                dispatchOp.handle_with_time_async('', global.get_current_time(),
+                                                  function(src, result) {
+                    src.handle_with_time_finish(result)});
+                break;
+            }
+            this.destroy();
+        }));
+    }
+};
+
+// Audio Video
+function AudioVideoNotification(source, dispatchOp, channel, contact, isVideo) {
+    this._init(source, dispatchOp, channel, contact, isVideo);
+}
+
+AudioVideoNotification.prototype = {
+    __proto__: MessageTray.Notification.prototype,
+
+    _init: function(source, dispatchOp, channel, contact, isVideo) {
+        let title = '';
+
+        if (isVideo)
+             /* translators: argument is a contact name like Alice for example. */
+            title = _("Video call from %s").format(contact.get_alias());
+        else
+             /* translators: argument is a contact name like Alice for example. */
+            title = _("Call from %s").format(contact.get_alias());
+
+        MessageTray.Notification.prototype._init.call(this,
+                                                      source,
+                                                      title,
+                                                      null,
+                                                      { customContent: true });
+        this.setResident(true);
+
+        this.addButton('reject', _("Reject"));
+        this.addButton('answer', _("Answer"));
+
+        this.connect('action-invoked', Lang.bind(this, function(self, action) {
+            switch (action) {
+            case 'reject':
+                dispatchOp.leave_channels_async(Tp.ChannelGroupChangeReason.NONE,
+                                                '', function(src, result) {
+                    src.leave_channels_finish(result)});
+                break;
+            case 'answer':
+                dispatchOp.handle_with_time_async('', global.get_current_time(),
+                                                  function(src, result) {
+                    src.handle_with_time_finish(result)});
+                break;
+            }
+            this.destroy();
+        }));
     }
 };
