@@ -119,6 +119,8 @@ WindowManager.prototype = {
         this.setKeybindingHandler('switch_to_workspace_down', Lang.bind(this, this._showWorkspaceSwitcher));
         this.setKeybindingHandler('switch_windows', Lang.bind(this, this._startAppSwitcher));
         this.setKeybindingHandler('switch_group', Lang.bind(this, this._startAppSwitcher));
+        this.setKeybindingHandler('switch_windows_backward', Lang.bind(this, this._startAppSwitcher));
+        this.setKeybindingHandler('switch_group_backward', Lang.bind(this, this._startAppSwitcher));
         this.setKeybindingHandler('switch_panels', Lang.bind(this, this._startA11ySwitcher));
 
         Main.overview.connect('showing', Lang.bind(this, function() {
@@ -180,7 +182,7 @@ WindowManager.prototype = {
          */
         this._minimizing.push(actor);
 
-        let primary = global.get_primary_monitor();
+        let primary = Main.layoutManager.primaryMonitor;
         let xDest = primary.x;
         if (St.Widget.get_default_direction() == St.TextDirection.RTL)
             xDest += primary.width;
@@ -534,7 +536,7 @@ WindowManager.prototype = {
 
         let tabPopup = new AltTab.AltTabPopup();
 
-        if (!tabPopup.show(backwards, binding == 'switch_group'))
+        if (!tabPopup.show(backwards, binding))
             tabPopup.destroy();
     },
 
